/*
 * Library interface to Linux x86 Performance-Monitoring Counters driver.
 *
 * Copyright (C) 1999  Mikael Pettersson
 *
 * Todo:
 * - Although the kernel will allow us to use the RDPMC instruction,
 *   we do not do so yet. The problem is that the kernel driver
 *   maintains correct PMCs _and_ TSC for our process. In user-space,
 *   we cannot maintain a correct TSC since the TSC is global.
 *
 *   My plan is to have the kernel driver export the perfctr_state
 *   through an mmap():able page. This would both obsolete the
 *   PERFCTR_SAMPLE command, and allow us to maintain current PMCs
 *   and TSC in user-space.
 */
#include "asm/perfctr.h"
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <errno.h>
#include "libperfctr.h"

static int perfctr_fd = -1;	/* to force initialization */

#define CHECK_INIT() \
do { \
  if( perfctr_fd < 0 && perfctr_init() < 0 ) \
    return -1; \
} while(0)

static int perfctr_init(void)
{
    if( (perfctr_fd = open("/dev/perfctr", O_RDONLY)) < 0 ) {
	return -1;
    }
    return 0;
}

int perfctr_info(struct perfctr_info *info)
{
    CHECK_INIT();
    return ioctl(perfctr_fd, PERFCTR_INFO, info);
}

int perfctr_control(struct perfctr_control *control)
{
    CHECK_INIT();
    return ioctl(perfctr_fd, PERFCTR_CONTROL, control);
}

int perfctr_sample(struct perfctr_state *state)
{
    CHECK_INIT();
    /* in the future we'd like to use RDPMC and RDTSC here instead */
    return ioctl(perfctr_fd, PERFCTR_SAMPLE, state);
}
